import pytest
from pyticketswitch import exceptions
from pyticketswitch.event import Event
from dateutil.tz import tzoffset


@pytest.fixture
def data():
    return {
        'meta_event_component_events': {
            'event': [{
                'event_id': 'META123'
            }]
        },
        'event_id': 'ABC1',
        'event_status': 'live',
        'event_type': 'simple_ticket',
        'source_desc': 'Super Awesome Ticketer',
        'venue_desc': 'Top Notch Theater',
        'venue_code': 'VENUECODE',
        'area_code': 'AREACODE',
        'lingo_data': {
            'lingo_code': 'theatre'
        },


        'classes': {
            'theatre': 'Theatre', 'amazeballs': 'Amazeballs'
        },

        #TODO: don't actually know what filters look like yet...
        'filters': [],

        'custom_fields': [
            {'custom_field_name': 'foo_bar'},
            {'custom_field_name': 'lol_beans'},
        ],

        'date_range_start': {
            'iso8601_date_and_time': '2016-07-21T23:57:15+01:00',
        },
        'date_range_end': {
            'iso8601_date_and_time': '2017-09-03T11:30:40+02:00',
        },

        'postcode': 'W6 8DL',
        'city_desc': 'London',
        'country_desc': 'United Kingdom',
        'country_code': 'uk',

        'geo_data': {
            'latitude': 51.49691253,
            'longitude': -0.17223274,
        },

        'max_running_time': 120,
        'min_running_time': 90,

        'has_no_perfs': True,
        'show_perf_time': True,
        'is_seated': True,
        'need_departure_date': True,
        'need_duration': True,
        'need_performance': True,

        'event_upsell_list': {
            'event_id': ['DEF2', 'GHI3', 'JKL4'],
        },
        'cost_range': {
            'max_seatprice': 47,
            'max_surcharge': 0,
            'min_seatprice': 18,
            'min_surcharge': 0,
            'no_singles_cost_range': {
                'max_seatprice': 47,
                'max_surcharge': 0,
                'min_seatprice': 18,
                'min_surcharge': 0,
                'quantity_options': {
                    'valid_quantity_bitmask': 126
                },
                'range_currency_code': 'gbp',
            },
            'quantity_options': {
                'valid_quantity_bitmask': 126
            },
            'range_currency_code': 'gbp',
        },
        'cost_range_details': {
            'ticket_type': [
                {
                    'ticket_type_desc': 'Grand Circle',
                    'price_band': [
                        {
                            'price_band_desc': '',
                            'cost_range': {
                                'quantity_options': {
                                    'valid_quantity_mask': 2046
                                },
                                'max_surcharge': 11.65,
                                'max_seatprice': 59.5,
                                'range_currency_code': 'gbp',
                                'min_surcharge': 11.25,
                                'no_singles_cost_range': {
                                    'quantity_options': {
                                        'valid_quantity_mask': 2046
                                    },
                                    'max_surcharge': 11.65,
                                    'max_seatprice': 59.5,
                                    'range_currency_code': 'gbp',
                                    'min_surcharge': 11.25,
                                    'min_seatprice': 57.5
                                },
                                'min_seatprice': 57.5
                            },
                            'price_band_code': 'B'
                        },
                    ],
                },
            ],
        },
        'structured_info': {
            'content': {
                'name': 'name',
                'value': 'value',
                'value_html': '<p>value</p>',
            }
        },
        'media': {
            'media_asset': [{
                'caption_html': '',
                'name': 'landscape',
                'secure_complete_url': 'https://d1wx4w35ubmdix.cloudfront.net/shared/event_media/cropper_cloud/24/249fdb60b56ef4217a6049a1ed770f13552bca9c.jpg',
                'supports_http': True,
                'caption': '',
                'host': 'd1wx4w35ubmdix.cloudfront.net',
                'supports_https': True,
                'path': '/shared/event_media/cropper_cloud/24/249fdb60b56ef4217a6049a1ed770f13552bca9c.jpg',
                'insecure_complete_url': 'http://d1wx4w35ubmdix.cloudfront.net/shared/event_media/cropper_cloud/24/249fdb60b56ef4217a6049a1ed770f13552bca9c.jpg'
            }]
        },
        'video_iframe': {
            'video_iframe_host': 'www.youtube.com',
            'video_iframe_supports_http': True,
            'video_iframe_supports_https': True,
            'video_iframe_path': '/embed/-pgZtzDj_7o?list=PL59A1A57EF510F502',
            'video_iframe_caption': '',
            'video_iframe_url_when_insecure': 'http://www.youtube.com/embed/-pgZtzDj_7o?list=PL59A1A57EF510F502',
            'video_iframe_width': 560,
            'video_iframe_height': 315,
            'video_iframe_url_when_secure': 'https://www.youtube.com/embed/-pgZtzDj_7o?list=PL59A1A57EF510F502',
            'video_iframe_caption_html': ''
        },
        'reviews': {
            'review': [
                {
                    'review_body': 'the finest mise-en-sc\xe8ne in the West End',
                    'review_iso8601_date_and_time': '2013-03-30T12:00:00Z',
                    'star_rating': 4,
                    'review_lang': 'en',
                    'review_title': '',
                    'is_user_review': False,
                    'review_date_desc': 'Sat, 30th March 2013',
                    'review_time_desc': '12.00 PM',
                    'review_author': "What's on Stage",
                    'review_original_url': 'http://www.whatsonstage.com/index.php?pg=207&story=E01822737494'
                }
            ]
        },
        'critic_review_percent': 100,
        'avail_details': {
            'ticket_type': [{
                'ticket_type_desc': 'Grand Circle',
                'price_band': [{
                    'price_band_desc': '',
                    'price_band_code': 'B',
                    'avail_detail': [{
                        'available_weekdays': {
                            'wed': True,
                            'sun': True,
                            'fri': True,
                            'tue': True,
                            'mon': False,
                            'thu': True,
                            'sat': True
                        },
                        'available_dates': {
                            'last_yyyymmdd': '20161002',
                            'first_yyyymmdd': '20160906',
                            'nov': {
                                'day_30': False,
                                'day_18': False,
                            },
                            'oct': {
                                'day_30': True,
                                'day_31': False,
                                'day_18': True,
                            },
                        },
                        'avail_currency_code': 'gbp',
                        'seatprice': 57.5,
                        'surcharge': 11.25,
                        'quantity_options': {
                            'valid_quantity_mask': 2046
                        },
                        'day_mask': 21,
                        'seatprice': 57.5,
                        'surcharge': 11.25
                    }]
                }]
            }]
        },
        'valid_quantities': [1, 2, 3, 4],
    }


class TestEvent:

    BST = tzoffset('BST', 3600)
    DRC = tzoffset('DRC', 7200)

    def test_from_api_data(self, data):

        event = Event.from_api_data(data)
        assert event.id is 'ABC1'
        assert event.status == 'live'

        assert event.description is None

        assert event.source == 'Super Awesome Ticketer'
        assert event.event_type == 'simple_ticket'
        assert event.venue == 'Top Notch Theater'
        assert event.venue_code == 'VENUECODE'
        assert event.area_code == 'AREACODE'
        assert event.lingo_code == 'theatre'

        assert event.critic_review_percent == 100

        assert event.classes == {'theatre': 'Theatre', 'amazeballs': 'Amazeballs'}
        assert event.filters == []

        assert event.postcode == 'W6 8DL'
        assert event.city == 'London'
        assert event.country == 'United Kingdom'
        assert event.country_code == 'uk'
        assert event.latitude == 51.49691253
        assert event.longitude == -0.17223274

        assert event.max_running_time == 120
        assert event.min_running_time == 90

        assert event.show_performance_time is True
        assert event.has_performances is False
        assert event.is_seated is True
        assert event.needs_departure_date is True
        assert event.needs_duration is True
        assert event.needs_performance is True

        assert event.upsell_list == ['DEF2', 'GHI3', 'JKL4']

        assert len(event.content) == 1

        assert len(event.media) == 2
        assert len(event.reviews) == 1

        assert len(event.component_events) == 1

        assert len(event.fields) == 2
        assert event.raw == data

        assert event.valid_quantities == [1, 2, 3, 4]

    def test_from_api_data_with_no_event_id(self):
        data = {
            'foo': 'bar',
        }

        with pytest.raises(exceptions.IntegrityError):
            Event.from_api_data(data)

    def test_from_api_data_with_has_no_perfs(self):
        data = {
            'event_id': 'ABC1',
            'has_no_perfs': True,
        }

        event = Event.from_api_data(data)

        assert event.has_performances is False

    def test_from_api_data_with_is_add_on(self):
        data = {
            'event_id': 'ABC2',
            'is_add_on': True,
        }

        event = Event.from_api_data(data)

        assert event.is_add_on

    def test_from_api_data_with_is_auto_quantity_add_on(self):
        data = {
            'event_id': 'ABC2',
            'is_auto_quantity_add_on': True,
        }

        event = Event.from_api_data(data)

        assert event.is_auto_quantity_add_on

    def test_from_events_by_id_api_data(self, data):
        raw_data = {
            'event': data,
            'add_ons': [
                {
                    'event_id': 'FOO',
                    'event_desc': 'Foo Test',
                },
            ],
            'upsells': [
                {
                    'event_id': 'BAR',
                    'event_desc': 'Bar Test',
                },
            ],
            'venue_is_enforced': False,
            'valid_quantities': [2, 4, 6, 8]
        }

        event = Event.from_events_by_id_api_data(raw_data)

        assert event.addon_events
        assert event.upsell_events
        assert event.venue_is_enforced is False
        assert event.valid_quantities == [2, 4, 6, 8]



import six

from pyticketswitch.currency import Currency, CurrencyMeta


class TestCurrency:

    def test_from_api_data(self):

        data = {
            'currency_factor': 100,
            'currency_places': 2,
            'currency_post_symbol': '\xa4',
            'currency_pre_symbol': '\xa3',
            'currency_code': 'gbp',
            'currency_number': 826,
        }
        currency = Currency.from_api_data(data)

        assert currency.factor == 100
        assert currency.places == 2
        assert currency.post_symbol == '\xa4'
        assert currency.pre_symbol == '\xa3'
        assert currency.code == 'gbp'
        assert currency.number == 826

    def test_eq(self):
        currency_one = Currency(
            code='usd', factor=321, places=2, number=123, pre_symbol='$',
            post_symbol='USD'
        )

        currency_two = Currency(
            code='usd', factor=321, places=2, number=123, pre_symbol='$',
            post_symbol='USD'
        )

        assert currency_one == currency_two

    def test_neq_code(self):
        currency_one = Currency('gbp')
        currency_two = Currency('usd')

        assert currency_one != currency_two

    def test_neq_factor(self):
        currency_one = Currency('gbp', factor=123)
        currency_two = Currency('gbp', factor=321)

        assert currency_one != currency_two

    def test_neq_number(self):
        currency_one = Currency('gbp', number=123)
        currency_two = Currency('gbp', number=321)

        assert currency_one != currency_two

    def test_neq_places(self):
        currency_one = Currency('gbp', places=3)
        currency_two = Currency('gbp', places=2)

        assert currency_one != currency_two

    def test_neq_pre_symbol(self):
        currency_one = Currency('gbp', pre_symbol='$')
        currency_two = Currency('gbp', pre_symbol='@')

        assert currency_one != currency_two

    def test_neq_post_symbol(self):
        currency_one = Currency('gbp', post_symbol='$')
        currency_two = Currency('gbp', post_symbol='@')

        assert currency_one != currency_two

    def test_neq_none(self):
        currency = Currency('gbp')
        assert currency != None  # noqa

    def test_price_from_string_with_post_symbol(self):

        currency = Currency(
            'bhd',
            places=3,
            pre_symbol=None,
            post_symbol='BD',
        )

        price = currency.price_as_string(13.1)
        assert price == '13.100BD'

    def test_price_from_string_with_pre_symbol(self):

        currency = Currency(
            'fjd',
            places=2,
            pre_symbol='$',
            post_symbol=None,
        )

        price = currency.price_as_string(13.1)
        assert price == '$13.10'

    def test_price_from_string_with_ascii_encoded(self):
        currency = Currency(
            'edw',
            places=2,
            pre_symbol=u'\xa3',
            post_symbol=None,
        )
        price = currency.price_as_string(13.1)
        assert price == six.text_type(u'\xa313.10')

    def test_repr(self):
        currency = Currency('fjd')
        assert repr(currency) == '<Currency fjd>'


class TestCurrencyMeta:

    def test_from_api_data(self):

        data = {
            'currency_details': {
                'usd': {
                    "currency_code": "usd",
                },
                'gbp': {
                    "currency_code": "gbp",
                }
            },
            'currency_code': "gbp",
            'desired_currency_code': "usd",
        }

        meta = CurrencyMeta.from_api_data(data)

        assert isinstance(meta, CurrencyMeta)

        gbp = meta.currencies['gbp']
        assert isinstance(gbp, Currency)
        assert gbp.code == 'gbp'

        usd = meta.currencies['usd']
        assert isinstance(usd, Currency)
        assert usd.code == 'usd'

        assert meta.desired_currency_code == 'usd'
        assert meta.default_currency_code == 'gbp'

    def test_from_api_data_with_no_currency_data(self):

        data = {
            'foo': 'bar'
        }

        meta = CurrencyMeta.from_api_data(data)

        assert meta.currencies == {}

    def test_get_currency(self):
        data = {
            'currency_details': {
            },
            'desired_currency_code': "usd",
        }

        meta = CurrencyMeta.from_api_data(data)
        currency = meta.get_currency()
        assert currency is None

    def test_get_desired_currency_with_no_data(self):
        data = {
            'currency_details': {
            },
            'desired_currency_code': "usd",
        }

        meta = CurrencyMeta.from_api_data(data)
        currency = meta.get_desired_currency()
        assert currency is None

    def test_get_desired_currency(self):
        data = {
            'currency_details': {
                'usd': {
                    "currency_code": "usd",
                },
                'gbp': {
                    "currency_code": "gbp",
                }
            },
            'currency_code': "gbp",
            'desired_currency_code': "usd",
        }

        meta = CurrencyMeta.from_api_data(data)
        currency = meta.get_desired_currency()
        assert currency.code == 'usd'



import datetime
from pyticketswitch.performance import Performance, PerformanceMeta


class TestPerformance:

    def test_from_api_data(self):

        data = {
            'iso8601_date_and_time': '2016-10-05T14:30:00+01:00',
            'event_id': '25DR',
            'time_desc': '2.30 PM',
            'date_desc': 'Wed, 5th October 2016',
            'is_limited': False,
            'is_ghost': False,
            'perf_id': '25DR-52O',
            'perf_name': 'ABC123',
            'has_pool_seats': True,
            'cached_max_seats': 10,
            'running_time': 200,
            'cost_range': {
                'quantity_options': {
                    'valid_quantity_mask': 2046
                },
                'max_surcharge': 11.65,
                'max_seatprice': 59.5,
                'range_currency': {
                    'currency_factor': 100,
                    'currency_places': 2,
                    'currency_post_symbol': '',
                    'currency_number': 826,
                    'currency_pre_symbol': '\xa3',
                    'currency_code': 'gbp'
                },
                'min_surcharge': 11.25,
                'no_singles_cost_range': {
                    'quantity_options': {
                        'valid_quantity_mask': 2046
                    },
                    'max_surcharge': 11.65,
                    'max_seatprice': 69.5,
                    'range_currency': {
                        'currency_factor': 100,
                        'currency_places': 2,
                        'currency_post_symbol': '',
                        'currency_number': 826,
                        'currency_pre_symbol': '\xa3',
                        'currency_code': 'gbp'
                    },
                    'min_surcharge': 11.25,
                    'min_seatprice': 57.5
                },
                'min_seatprice': 57.5
            },
            'avail_details': {
                'ticket_type': [
                    {
                        'price_band': [
                            {
                                'avail_detail': [
                                    {
                                        'avail_currency': {
                                            'currency_code': 'gbp'
                                        },
                                        'cached_number_available': 6,
                                        'quantity_options': {
                                            'valid_quantity_bitmask': 126
                                        },
                                        'seatprice': 47,
                                        'surcharge': 0
                                    }
                                ],
                                'price_band_code': 'A',
                                'price_band_desc': ''
                            }
                        ],
                        'ticket_type_code': 'BALCONY',
                        'ticket_type_desc': 'Balcony'
                    },
                    {
                        'price_band': [
                            {
                                'avail_detail': [
                                    {
                                        'avail_currency': {
                                            'currency_code': 'gbp'
                                        },
                                        'cached_number_available': 6,
                                        'quantity_options': {
                                            'valid_quantity_bitmask': 126
                                        },
                                        'seatprice': 35,
                                        'surcharge': 0
                                    }
                                ],
                                'price_band_code': 'A',
                                'price_band_desc': ''
                            },
                            {
                                'avail_detail': [
                                    {
                                        'avail_currency': {
                                            'currency_code': 'gbp'
                                        },
                                        'cached_number_available': 6,
                                        'quantity_options': {
                                            'valid_quantity_bitmask': 126
                                        },
                                        'seatprice': 30,
                                        'surcharge': 0
                                    }
                                ],
                                'price_band_code': 'B',
                                'price_band_desc': ''
                            },
                            {
                                'avail_detail': [
                                    {
                                        'avail_currency': {
                                            'currency_code': 'gbp'
                                        },
                                        'cached_number_available': 6,
                                        'quantity_options': {
                                            'valid_quantity_bitmask': 126
                                        },
                                        'seatprice': 25,
                                        'surcharge': 0
                                    }
                                ],
                                'price_band_code': 'C',
                                'price_band_desc': ''
                            }
                        ],
                        'ticket_type_code': "CIRCLE",
                        'ticket_type_desc': "Upper circle"
                    },
                    {
                        'price_band': [
                            {
                                'avail_detail': [
                                    {
                                        'avail_currency': {
                                            'currency_code': 'gbp'
                                        },
                                        'cached_number_available': 6,
                                        'quantity_options': {
                                            'valid_quantity_bitmask': 126
                                        },
                                        'seatprice': 21,
                                        'surcharge': 0
                                    }
                                ],
                                'price_band_code': 'A',
                                'price_band_desc': ''
                            },
                            {
                                'avail_detail': [
                                    {
                                        'avail_currency': {
                                            'currency_code': 'gbp'
                                        },
                                        'cached_number_available': 3,
                                        'quantity_options': {
                                            'valid_quantity_bitmask': 14
                                        },
                                        'seatprice': 18,
                                        'surcharge': 0
                                    }
                                ],
                                'price_band_code': 'B',
                                'price_band_desc': ''
                            }
                        ],
                        'ticket_type_code': 'STALLS',
                        'ticket_type_desc': 'Stalls'
                    }
                ]
            }
        }

        performance = Performance.from_api_data(data)
        assert performance.id == '25DR-52O'
        assert performance.event_id == '25DR'
        assert performance.date_time.year == 2016
        assert performance.date_time.month == 10
        assert performance.date_time.day == 5
        assert performance.date_time.hour == 14
        assert performance.has_pool_seats is True
        assert performance.cached_max_seats == 10
        assert performance.cost_range.max_seatprice == 59.5
        assert performance.no_singles_cost_range.max_seatprice == 69.5
        assert performance.is_ghost is False
        assert performance.name == 'ABC123'
        assert performance.running_time == 200

        assert len(performance.availability_details) == 6

    def test_repr_with_date(self):
        performance = Performance(
            'ABC1-23',
            'ABC1',
            date_time=datetime.datetime(2017, 2, 8, 14, 8, 0),
        )
        assert repr(performance) == u'<Performance ABC1-23: 2017-02-08T14:08:00>'

    def test_repr_without_date(self):
        performance = Performance('ABC1-23', 'ABC1')
        assert repr(performance) == u'<Performance ABC1-23>'


class TestPerformanceMeta:

    def test_from_api_data(self):
        data = {
            'autoselect_this_performance': True,
            'results': {
                'has_perf_names': True,
                'performance': [
                    {'perf_id': '6IF-B1H', 'event_id': '6IF'}
                ],
                'paging_status': {
                    'total_unpaged_results': 1,
                },
            },
            'currency_code': 'gbp',
            'currency_details': {
                'gbp': {
                    'currency_code': 'gbp',
                }
            }
        }

        meta = PerformanceMeta.from_api_data(data)

        assert meta.auto_select is True
        assert meta.has_names is True
        assert meta.total_results == 1
        assert 'gbp' in meta.currencies
        assert meta.default_currency_code == 'gbp'



import pytest
from pyticketswitch.exceptions import InvalidParametersError
from pyticketswitch.address import Address
from pyticketswitch.payment_methods import (
    PaymentMethod, CardDetails, RedirectionDetails, StripeDetails, CiderDetails
)


class TestCardDetails:

    def test_is_payment_method(self):
        card_details = CardDetails('411111111111111')
        assert isinstance(card_details, PaymentMethod)

    def test_as_api_parameters(self):
        card_details = CardDetails(
            '4111 1111 1111 1111',
            expiry_month=5,
            expiry_year=45,
            start_month=3,
            start_year=19,
            ccv2=123,
            issue_number=7,
        )

        params = card_details.as_api_parameters()

        assert params == {
            'card_number': '4111 1111 1111 1111',
            'expiry_date': '0545',
            'start_date': '0319',
            'cv_two': 123,
            'issue_number': 7,
        }

    def test_as_api_parameters_without_expiry_date(self):
        card_details = CardDetails('4111 1111 1111 1111')
        with pytest.raises(InvalidParametersError):
            card_details.as_api_parameters()

    def test_as_api_parameters_without_expiry_month(self):
        card_details = CardDetails('4111 1111 1111 1111', expiry_year=45)
        with pytest.raises(InvalidParametersError):
            card_details.as_api_parameters()

    def test_as_api_parameters_without_expiry_year(self):
        card_details = CardDetails('4111 1111 1111 1111', expiry_month=5)
        with pytest.raises(InvalidParametersError):
            card_details.as_api_parameters()

    def test_as_api_parameters_without_extras(self):
        card_details = CardDetails(
            '4111 1111 1111 1111',
            expiry_month=5,
            expiry_year=45,
        )

        params = card_details.as_api_parameters()

        assert params == {
            'card_number': '4111 1111 1111 1111',
            'expiry_date': '0545',
        }

    def test_as_api_parameters_with_start_month_without_start_year(self):
        card_details = CardDetails(
            '4111 1111 1111 1111',
            expiry_month=5,
            expiry_year=45,
            start_month=3,
        )

        with pytest.raises(InvalidParametersError):
            card_details.as_api_parameters()

    def test_as_api_parameters_with_start_year_without_start_month(self):
        card_details = CardDetails(
            '4111 1111 1111 1111',
            expiry_month=5,
            expiry_year=45,
            start_year=19,
        )

        with pytest.raises(InvalidParametersError):
            card_details.as_api_parameters()

    def test_as_api_parameters_with_billing_address(self):
        billing_address = Address(
            lines=['1303 Boulder Lane', 'Landslide district'],
            country_code='us',
            county='Louisana',
            town='Bedrock',
            post_code='70777',
        )
        card_details = CardDetails(
            '4111 1111 1111 1111',
            expiry_month=5,
            expiry_year=45,
            billing_address=billing_address,
        )

        params = card_details.as_api_parameters()

        assert params == {
            'card_number': '4111 1111 1111 1111',
            'expiry_date': '0545',
            'billing_address_line_one': '1303 Boulder Lane',
            'billing_address_line_two': 'Landslide district',
            'billing_country_code': 'us',
            'billing_county': 'Louisana',
            'billing_town': 'Bedrock',
            'billing_postcode': '70777',
        }

    def test_as_api_parameters_with_return_url_and_token(self):
        card_details = CardDetails(
            '4111 1111 1111 1111',
            expiry_month=5,
            expiry_year=45,
            return_url='https://acmetickets.com/checkout/',
            remote_site='acmetickets.com',
            return_token='abc123',
        )

        params = card_details.as_api_parameters()

        assert params == {
            'card_number': '4111 1111 1111 1111',
            'expiry_date': '0545',
            'return_url': 'https://acmetickets.com/checkout/',
            'remote_site': 'acmetickets.com',
            'return_token': 'abc123',
        }

    def test_as_api_parameters_with_user_agent_header(self):
        card_details = CardDetails(
            '4111 1111 1111 1111',
            expiry_month=5,
            expiry_year=45,
            user_agent='iceweasle',
        )

        params = card_details.as_api_parameters()

        assert params == {
            'card_number': '4111 1111 1111 1111',
            'expiry_date': '0545',
            'client_http_user_agent': 'iceweasle',
        }

    def test_as_api_parameters_with_accept_header(self):
        card_details = CardDetails(
            '4111 1111 1111 1111',
            expiry_month=5,
            expiry_year=45,
            accept='application/json',
        )

        params = card_details.as_api_parameters()

        assert params == {
            'card_number': '4111 1111 1111 1111',
            'expiry_date': '0545',
            'client_http_accept': 'application/json',
        }

    def test_as_api_parameters_with_long_expiry_start_year(self):
        card_details = CardDetails(
            '4111 1111 1111 1111',
            expiry_month=5,
            expiry_year=2017,
            start_month=3,
            start_year=1943,
        )

        params = card_details.as_api_parameters()

        assert params == {
            'card_number': '4111 1111 1111 1111',
            'expiry_date': '0517',
            'start_date': '0343',
        }


class TestRedirectionDetails:

    def test_is_payment_method(self):
        redirect_details = RedirectionDetails(
            'abc123',
            'https://foobar.com/payment/abc123',
            'iceweasle 9000',
            'application/json',
            'foobar.com',
        )
        assert isinstance(redirect_details, PaymentMethod)

    def test_as_api_parameters(self):
        redirect_details = RedirectionDetails(
            'abc123',
            'https://foobar.com/payment/abc123',
            'iceweasle 9000',
            'application/json',
            'foobar.com',
        )

        params = redirect_details.as_api_parameters()

        assert params == {
            'return_token': 'abc123',
            'return_url': 'https://foobar.com/payment/abc123',
            'client_http_user_agent': 'iceweasle 9000',
            'client_http_accept': 'application/json',
            'remote_site': 'foobar.com'
        }


class TestStripeDetails:

    def test_is_payment_method(self):
        stripe_details = StripeDetails({'foo': 'abc123', 'bar': 'def456'})
        assert isinstance(stripe_details, PaymentMethod)

    def test_as_api_parameters(self):
        stripe_details = StripeDetails({'foo': 'abc123', 'bar': 'def456'})
        params = stripe_details.as_api_parameters()
        assert params == {
            'foo_callback/stripeToken': 'abc123',
            'bar_callback/stripeToken': 'def456',
        }


class TestCiderDetails:

    def test_is_payment_method(self):
        cider_details = CiderDetails({'beep': 'bop'}, ["test"])
        assert isinstance(cider_details, PaymentMethod)

    def test_as_api_parameters(self):
        cider_details = CiderDetails({'beep': 'boop'}, ["test", "bip"])
        params = cider_details.as_api_parameters()
        assert params == {
            'test_callback/beep': 'boop',
            'bip_callback/beep': 'boop',
        }



from pyticketswitch.cost_range import CostRange, CostRangeDetails
from pyticketswitch.offer import Offer


class TestCostRange:

    def test_from_api_data(self):
        data = {
            "best_value_offer": {
                "absolute_saving": 10,
            },
            "max_saving_offer": {
                "absolute_saving": 8,
            },
            "max_seatprice": 149.5,
            "max_surcharge": 29.65,
            "min_cost_offer": {
                "absolute_saving": 7,
            },
            "min_seatprice": 37.5,
            "min_surcharge": 7.25,
            "range_currency_code": "usd",
            "top_price_offer": {
                "absolute_saving": 9,
            },
            "max_combined_combined_tax_component": 35.0,
            "max_combined_surcharge_tax_sub_component": 0.0,
            "min_combined_combined_tax_component": 35.0,
            "min_combined_surcharge_tax_sub_component": 0.0,
            "valid_quantities": [1, 2, 3, 4]
        }
        cost_range = CostRange.from_api_data(data)

        assert cost_range.max_surcharge == 29.65
        assert cost_range.max_seatprice == 149.5
        assert cost_range.min_surcharge == 7.25
        assert cost_range.min_seatprice == 37.5

        assert cost_range.top_price_offer.absolute_saving == 9.0
        assert cost_range.min_cost_offer.absolute_saving == 7.0
        assert cost_range.max_saving_offer.absolute_saving == 8.0
        assert cost_range.best_value_offer.absolute_saving == 10.0

        assert cost_range.currency == 'usd'
        assert cost_range.valid_quantities == [1, 2, 3, 4]

        assert cost_range.max_combined_combined_tax_component == 35.0
        assert cost_range.max_combined_surcharge_tax_sub == 0.0
        assert cost_range.min_combined_combined_tax_component == 35.0
        assert cost_range.min_combined_surcharge_tax_sub == 0.0

    def test_has_offer_with_no_offers(self):
        cost_range = CostRange()
        assert cost_range.has_offer() is False

    def test_has_offer_with_best_value_offer(self):
        offer = Offer()
        cost_range = CostRange(best_value_offer=offer)
        assert cost_range.has_offer() is True

    def test_has_offer_with_max_saving_offer(self):
        offer = Offer()
        cost_range = CostRange(max_saving_offer=offer)
        assert cost_range.has_offer() is True

    def test_has_offer_with_min_cost_offer(self):
        offer = Offer()
        cost_range = CostRange(min_cost_offer=offer)
        assert cost_range.has_offer() is True

    def test_has_offer_with_top_price_offer(self):
        offer = Offer()
        cost_range = CostRange(top_price_offer=offer)
        assert cost_range.has_offer() is True

    def test_get_min_combined_price(self):
        cost_range = CostRange(min_seatprice=10, min_surcharge=5)
        assert cost_range.get_min_combined_price() == 15.0

    def test_get_max_combined_price(self):
        cost_range = CostRange(max_seatprice=22, max_surcharge=8)
        assert cost_range.get_max_combined_price() == 30.0

    def test_has_tax_with_combined_tax_components(self):
        cost_range = CostRange(
            max_combined_combined_tax_component=35.0,
            min_combined_combined_tax_component=0.0,
        )
        assert cost_range.has_tax() is True

    def test_has_tax_with_surcharge_tax_sub_components(self):
        cost_range = CostRange(
            max_combined_surcharge_tax_sub=35.0,
            min_combined_surcharge_tax_sub=0.0,
        )
        assert cost_range.has_tax() is True


class TestCostRangeDetails:

    def test_from_api_data_flattens_ticket_types_and_price_bands(self):

        data = {
            'ticket_type': [
                {
                    'ticket_type_code': 'FOO',
                    'ticket_type_desc': 'Foo',
                    'price_band': [
                        {
                            'price_band_code': 'PLB',
                            'price_band_desc': 'Plebians',
                            'cost_range': {
                                'max_surcharge': 29.65,
                                'max_seatprice': 149.5,
                            },
                        },
                        {
                            'price_band_code': 'TRG',
                            'price_band_desc': 'Troglodites',
                            'cost_range': {
                                'max_surcharge': 29.65,
                                'max_seatprice': 149.5,
                            },
                        }
                    ]
                },
                {
                    'ticket_type_code': 'BAR',
                    'ticket_type_desc': 'Bar',
                    'price_band': [
                        {
                            'price_band_code': 'VIP',
                            'price_band_desc': 'Very Important People',
                            'cost_range': {
                                'max_surcharge': 29.65,
                                'max_seatprice': 149.5,
                            },
                        },
                        {
                            'price_band_code': 'CBH',
                            'price_band_desc': 'Chinese Billionaire Heiress',
                            'cost_range': {
                                'max_surcharge': 29.65,
                                'max_seatprice': 149.5,
                            },
                        }
                    ]
                }
            ]
        }

        details = CostRangeDetails.from_api_data(data)
        assert len(details) == 4

        assert details[0].ticket_type_code == 'FOO'
        assert details[0].ticket_type_description == 'Foo'
        assert details[0].price_band_code == 'PLB'
        assert details[0].price_band_description == 'Plebians'

        assert details[1].ticket_type_code == 'FOO'
        assert details[1].ticket_type_description == 'Foo'
        assert details[1].price_band_code == 'TRG'
        assert details[1].price_band_description == 'Troglodites'

        assert details[2].ticket_type_code == 'BAR'
        assert details[2].ticket_type_description == 'Bar'
        assert details[2].price_band_code == 'VIP'
        assert details[2].price_band_description == 'Very Important People'

        assert details[3].ticket_type_code == 'BAR'
        assert details[3].ticket_type_description == 'Bar'
        assert details[3].price_band_code == 'CBH'
        assert details[3].price_band_description == 'Chinese Billionaire Heiress'

    def test_from_api_data_adds_cost_range(self):
        data = {
            'ticket_type': [
                {
                    'ticket_type_code': 'FOO',
                    'ticket_type_desc': 'Foo',
                    'price_band': [
                        {
                            'price_band_code': 'PLB',
                            'price_band_desc': 'Plebians',
                            'cost_range': {
                                'max_surcharge': 29.65,
                                'max_seatprice': 149.5,
                            },
                        },
                    ]
                },
            ]
        }

        details = CostRangeDetails.from_api_data(data)
        assert len(details) == 1
        assert isinstance(details[0].cost_range, CostRange)

    def test_from_api_data_adds_cost_range_no_singles(self):
        data = {
            'ticket_type': [
                {
                    'ticket_type_code': 'FOO',
                    'ticket_type_desc': 'Foo',
                    'price_band': [
                        {
                            'price_band_code': 'PLB',
                            'price_band_desc': 'Plebians',
                            'cost_range': {
                                'no_singles_cost_range': {
                                    'max_surcharge': 29.65,
                                    'max_seatprice': 149.5,
                                }
                            },
                        },
                    ]
                },
            ]
        }

        details = CostRangeDetails.from_api_data(data)
        assert len(details) == 1
        assert isinstance(details[0].cost_range_no_singles, CostRange)

    def test_from_api_data_ignores_data_with_no_cost_ranges(self):
        data = {
            'ticket_type': [
                {
                    'ticket_type_code': 'FOO',
                    'ticket_type_desc': 'Foo',
                    'price_band': [
                        {
                            'price_band_code': 'PLB',
                            'price_band_desc': 'Plebians',
                        },
                    ]
                },
            ]
        }

        details = CostRangeDetails.from_api_data(data)
        assert len(details) == 0



from pyticketswitch.purchase_result import PurchaseResult


class TestPurchaseResult:

    def test_from_api_data(self):

        data = {
            "failed_3d_secure": True,
            "failed_avs": True,
            "failed_cv_two": True,
            "purchase_error": "too much donk",
            "success": True,
            "failure_reason": "unknown",
            "internal_error_comment": "Even more error info",
            "is_partial": True,
            "is_semi_credit": False,
        }

        purchase_result = PurchaseResult.from_api_data(data)

        assert purchase_result.success is True
        assert purchase_result.failed_3d_secure is True
        assert purchase_result.failed_avs is True
        assert purchase_result.failed_cv_two is True
        assert purchase_result.error == 'too much donk'
        assert purchase_result.failure_reason == 'unknown'
        assert purchase_result.internal_error_comment == 'Even more error info'
        assert purchase_result.is_partial
        assert not purchase_result.is_semi_credit



from pyticketswitch.address import Address


class TestAddress:

    def test_from_api_data(self):
        data = {
            'address_line_one': 'Metro Building',
            'address_line_two': '1 Butterwick',
            'country_code': 'uk',
            'county': 'London',
            'email_address': 'lol@beans.com',
            'home_phone': '020810101010101',
            'postcode': 'W6 8DL',
            'town': 'Hammersmith, London',
            'work_phone': '020801010101010'
        }

        address = Address.from_api_data(data)

        assert address.lines[0] == 'Metro Building'
        assert address.lines[1] == '1 Butterwick'
        assert address.town == 'Hammersmith, London'
        assert address.post_code == 'W6 8DL'
        assert address.country_code == 'uk'
        assert address.county == 'London'
        assert address.email == 'lol@beans.com'
        assert address.home_phone == '020810101010101'
        assert address.work_phone == '020801010101010'

    def test_as_api_billing_address_parameters(self):

        address = Address(
            lines=['1303 Boulder Lane', 'Landslide district'],
            country_code='us',
            county='Louisana',
            town='Bedrock',
            post_code='70777',
        )

        params = address.as_api_billing_address_parameters()

        assert params == {
            'billing_address_line_one': '1303 Boulder Lane',
            'billing_address_line_two': 'Landslide district',
            'billing_country_code': 'us',
            'billing_county': 'Louisana',
            'billing_town': 'Bedrock',
            'billing_postcode': '70777',
        }

    def test_as_api_billing_address_parameters_single(self):

        address = Address(
            lines=['1303 Boulder Lane'],
            country_code='us',
            county='Louisana',
            town='Bedrock',
            post_code='70777',
        )

        params = address.as_api_billing_address_parameters()

        assert params == {
            'billing_address_line_one': '1303 Boulder Lane',
            'billing_country_code': 'us',
            'billing_county': 'Louisana',
            'billing_town': 'Bedrock',
            'billing_postcode': '70777',
        }



from pyticketswitch.offer import Offer


class TestOffer:

    def test_from_api_data(self):
        data = {
            "absolute_saving": 9,
            "percentage_saving": 11,
            "full_seatprice": 75,
            "full_surcharge": 5,
            "offer_seatprice": 68,
            "offer_surcharge": 3,
        }
        offer = Offer.from_api_data(data)

        assert offer.absolute_saving == 9
        assert offer.percentage_saving == 11
        assert offer.original_seatprice == 75
        assert offer.original_surcharge == 5
        assert offer.seatprice == 68
        assert offer.surcharge == 3



from pyticketswitch.trolley import Trolley
from pyticketswitch.bundle import Bundle
from pyticketswitch.event import Event
from pyticketswitch.order import Order


class TestTrolley:

    def test_from_api_data_with_trolley_data(self):
        data = {
            'discarded_orders': [
                {'item_number': 3},
                {'item_number': 6},
            ],

            'trolley_contents': {
                'bundle': [
                    {'bundle_source_code': 'foo'},
                    {'bundle_source_code': 'bar'},
                ],
                "purchase_result": {'success': True},
            },
            'trolley_token': 'ABC123',
            'trolley_order_count': 8,
        }

        trolley = Trolley.from_api_data(data)

        assert trolley.token == 'ABC123'

        assert len(trolley.bundles) == 2
        assert trolley.bundles[0].source_code == 'foo'
        assert trolley.bundles[1].source_code == 'bar'

        assert len(trolley.discarded_orders) == 2
        assert trolley.discarded_orders[0].item == 3
        assert trolley.discarded_orders[1].item == 6
        assert trolley.order_count == 8
        assert trolley.purchase_result.success is True

    def test_from_api_data_with_reservation_data(self):
        data = {
            'discarded_orders': [
                {'item_number': 3},
                {'item_number': 6},
            ],

            'reserved_trolley': {
                'bundle': [
                    {'bundle_source_code': 'foo'},
                    {'bundle_source_code': 'bar'},
                ],
                'transaction_id': 'T1000-0000',
                'transaction_uuid': 'DEF456',
            },
        }

        trolley = Trolley.from_api_data(data)

        assert trolley.transaction_uuid == 'DEF456'
        assert trolley.transaction_id == 'T1000-0000'

        assert len(trolley.bundles) == 2
        assert trolley.bundles[0].source_code == 'foo'
        assert trolley.bundles[1].source_code == 'bar'

        assert len(trolley.discarded_orders) == 2
        assert trolley.discarded_orders[0].item == 3
        assert trolley.discarded_orders[1].item == 6

    def test_from_api_data_with_empty_trolley(self):
        data = {
            "discarded_orders": [],
            "input_contained_unavailable_order": True,
            "trolley_token": "abc123",
            "trolley_token_contents": {
                "trolley_bundle_count": 0,
                "trolley_order_count": 0
            }
        }

        trolley = Trolley.from_api_data(data)

        assert trolley.token == 'abc123'
        assert trolley.input_contained_unavailable_order is True


    def test_get_events(self):

        event_one = Event(id_='abc123')
        event_two = Event(id_='def456')
        event_three = Event(id_='ghi789')
        event_four = Event(id_='jlk012')

        bundle_one = Bundle(
            'tests',
            orders=[
                Order(item=1, event=event_one),
                Order(item=2, event=event_two),
            ]
        )
        bundle_two = Bundle(
            'tests_two',
            orders=[
                Order(item=3, event=event_three),
                Order(item=4, event=event_four),
            ]
        )

        trolley = Trolley(
            bundles=[bundle_one, bundle_two]
        )

        events = trolley.get_events()

        assert events == [event_one, event_two, event_three, event_four]

    def test_get_events_with_no_bundles(self):

        trolley = Trolley(bundles=None)

        events = trolley.get_events()

        assert events == []

    def test_get_event_ids(self):

        event_one = Event(id_='abc123')
        event_two = Event(id_='def456')
        event_three = Event(id_='ghi789')
        event_four = Event(id_='abc123')

        bundle_one = Bundle(
            'tests',
            orders=[
                Order(item=1, event=event_one),
                Order(item=2, event=event_two),
            ]
        )
        bundle_two = Bundle(
            'tests_two',
            orders=[
                Order(item=3, event=event_three),
                Order(item=4, event=event_four),
            ]
        )

        trolley = Trolley(
            bundles=[bundle_one, bundle_two]
        )

        events = trolley.get_event_ids()

        assert events == {'abc123', 'def456', 'ghi789'}

    def test_get_bundle(self):

        # state
        bundle_one = Bundle('tests')
        bundle_two = Bundle('tests_two')
        trolley = Trolley(bundles=[bundle_one, bundle_two])

        # action
        bundle = trolley.get_bundle('tests_two')

        # results
        assert bundle is bundle_two

    def test_get_bundle_when_none(self):

        # state
        trolley = Trolley(bundles=[])

        # action
        bundle = trolley.get_bundle('tests_two')

        # results
        assert bundle is None

    def test_get_bundle_when_no_match(self):

        # state
        bundle_one = Bundle('tests')
        bundle_two = Bundle('tests_two')
        trolley = Trolley(bundles=[bundle_one, bundle_two])

        # action
        bundle = trolley.get_bundle('tests_three')

        # results
        assert bundle is None

    def test_get_item(self):

        #state
        order_one = Order(1)
        order_two = Order(2)
        order_three = Order(3)
        order_four = Order(4)
        bundle_one = Bundle('tests', orders=[order_one, order_three])
        bundle_two = Bundle('tests_two', orders=[order_two, order_four])
        trolley = Trolley(bundles=[bundle_one, bundle_two])

        # results
        order = trolley.get_item(3)
        assert order is order_three

        # results
        order = trolley.get_item(2)
        assert order is order_two

    def test_get_orders(self):

        #state
        order_one = Order(1)
        order_two = Order(2)
        order_three = Order(3)
        order_four = Order(4)
        bundle_one = Bundle('tests', orders=[order_one, order_three])
        bundle_two = Bundle('tests_two', orders=[order_two, order_four])
        trolley = Trolley(bundles=[bundle_one, bundle_two])

        # results
        orders = trolley.get_orders()
        assert orders == [order_one, order_three, order_two, order_four]



import docs.conf
import pyticketswitch
import re


def test_versions():
    # check all the version numbers are consistent.
    print('pyticketswitch/init.py: {}'.format(pyticketswitch.__version__))
    print('docs/conf.py (version): {}'.format(docs.conf.version))
    print('docs/conf.py (release): {}'.format(docs.conf.release))

    setup = None
    with open('setup.py', 'r') as fh:
        match = re.search(r"\s*version\s*=\s*'([^']+)'", fh.read())
        if match:
            setup = match.group(1)
            print('setup.py: {}'.format(setup))
        else:
            print('setup.py: MISSING!')

    all_match = all(
        version == pyticketswitch.__version__
        for version in [docs.conf.version, docs.conf.release, setup]
    )
    assert all_match, 'Version numbers do not match pyticketswitch.__version__'

    # check that the change log has an entry for this version
    with open('CHANGELOG.md', 'r') as fh:
        match = re.search(
            "## \[{}\]".format(pyticketswitch.__version__), fh.read())
        assert match, 'no entry in CHANGELOG.md for current version {}'.format(
            pyticketswitch.__version__
        )
        print("CHANGELOG.md: {}".format(
            pyticketswitch.__version__
        ))



import pytest
import datetime
from pyticketswitch.availability import AvailabilityMeta, AvailabilityDetails
from pyticketswitch.currency import Currency


@pytest.fixture
def avail_details():
    kwargs = {
        'ticket_type': 'BALCONY',
        'ticket_type_description': 'Balcony',
        'price_band': 'A',
        'price_band_description': 'AYYY!',
        'seatprice': 47,
        'surcharge': 0,
        'currency': Currency(
            code='gbp',
            factor=100,
            places=2,
            number=826,
            pre_symbol='#',
            post_symbol='',
        ),
        'first_date': datetime.date(2016, 11, 29),
        'last_date': datetime.date(2017, 3, 27),
        'calendar_masks': {
            2016: {
                12: 1065287163,
                11: 805306368
            },
            2017: {
                2: 251526135,
                1: 2012209087,
                3: 117308407
            }
        },
        'weekday_mask': 63,
    }
    details = AvailabilityDetails(**kwargs)
    return details


@pytest.fixture
def data_meta():
    data = {
        "availability": {
            "ticket_type": [
                {
                    "price_band": [
                        {
                            "absolute_saving": 0,
                            "discount_code": "",
                            "discount_desc": "",
                            "is_alloc": False,
                            "is_offer": False,
                            "non_offer_sale_seatprice": 35,
                            "non_offer_sale_surcharge": 4,
                            "number_available": 6,
                            "percentage_saving": 0,
                            "price_band_code": "A",
                            "sale_seatprice": 35,
                            "sale_surcharge": 4,
                        },
                        {
                            "absolute_saving": 0,
                            "discount_code": "",
                            "discount_desc": "",
                            "is_alloc": False,
                            "is_offer": False,
                            "non_offer_sale_seatprice": 25,
                            "non_offer_sale_surcharge": 4,
                            "number_available": 6,
                            "percentage_saving": 0,
                            "price_band_code": "C",
                            "sale_seatprice": 25,
                            "sale_surcharge": 4,
                        }
                    ],
                    "ticket_type_code": "CIRCLE",
                    "ticket_type_desc": "Upper circle"
                },
                {
                    "price_band": [
                        {
                            "absolute_saving": 0,
                            "discount_code": "",
                            "discount_desc": "",
                            "is_alloc": False,
                            "is_offer": False,
                            "non_offer_sale_seatprice": 21,
                            "non_offer_sale_surcharge": 3,
                            "number_available": 6,
                            "percentage_saving": 0,
                            "price_band_code": "A",
                            "sale_seatprice": 21,
                            "sale_surcharge": 3
                        }
                    ],
                    "ticket_type_code": "STALLS",
                    "ticket_type_desc": "Stalls"
                }
            ]
        },
        "backend_is_broken": False,
        "backend_is_down": False,
        "backend_throttle_failed": False,
        "contiguous_seat_selection_only": True,
        "must_select_whole_seat_block": True,
        "currency_code": 'gbp',
        'currency_details': {
            "gbp": {
                "currency_code": "gbp",
            },
        },
        "max_bundle_size": 1,
        "source_code": "BackendCode",
        "valid_quantities": [
            1, 4, 6, 7
        ],
    }

    return data


class TestAvailabilityMeta:

    def test_from_api_data(self, data_meta):
        meta = AvailabilityMeta.from_api_data(data_meta)

        assert meta.contiguous_seat_selection_only is True
        assert meta.must_select_whole_seat_block is True
        assert meta.default_currency_code == 'gbp'
        assert meta.max_bundle_size == 1
        assert meta.valid_quantities == [1, 4, 6, 7]
        assert meta.backend_is_broken is False
        assert meta.backend_is_down is False
        assert meta.backend_throttle_failed is False
        assert meta.source_code == "BackendCode"

    def test_from_api_data_with_backend_issues(self):

        data = {
            'backend_is_broken': True,
            'backend_is_down': True,
            'backend_throttle_failed': True,
        }

        meta = AvailabilityMeta.from_api_data(data)

        assert meta.backend_is_broken is True
        assert meta.backend_is_down is True
        assert meta.backend_throttle_failed is True


class TestAvailabilityDetails:

    def test_from_api_data_flattens_ticket_types_and_price_bands(self):
        data = {
            'ticket_type': [
                {
                    'ticket_type_code': 'FOO',
                    'ticket_type_desc': 'Foo',
                    'price_band': [
                        {
                            'price_band_code': 'PLB',
                            'price_band_desc': 'Plebians',
                        },
                        {
                            'price_band_code': 'TRG',
                            'price_band_desc': 'Troglodites',
                        }
                    ]
                },
                {
                    'ticket_type_code': 'BAR',
                    'ticket_type_desc': 'Bar',
                    'price_band': [
                        {
                            'price_band_code': 'VIP',
                            'price_band_desc': 'Very Important People',
                        },
                        {
                            'price_band_code': 'CBH',
                            'price_band_desc': 'Chinese Billionaire Heiress',
                        }
                    ]
                }
            ]
        }

        details = AvailabilityDetails.from_api_data(data)
        assert len(details) == 4

        assert details[2].ticket_type == 'FOO'
        assert details[2].ticket_type_description == 'Foo'
        assert details[2].price_band == 'PLB'
        assert details[2].price_band_description == 'Plebians'

        assert details[3].ticket_type == 'FOO'
        assert details[3].ticket_type_description == 'Foo'
        assert details[3].price_band == 'TRG'
        assert details[3].price_band_description == 'Troglodites'

        assert details[0].ticket_type == 'BAR'
        assert details[0].ticket_type_description == 'Bar'
        assert details[0].price_band == 'VIP'
        assert details[0].price_band_description == 'Very Important People'

        assert details[1].ticket_type == 'BAR'
        assert details[1].ticket_type_description == 'Bar'
        assert details[1].price_band == 'CBH'
        assert details[1].price_band_description == 'Chinese Billionaire Heiress'

    def test_from_api_data_adds_prices_availability(self):
        data = {
            'ticket_type': [
                {
                    'ticket_type_code': 'FOO',
                    'ticket_type_desc': 'Foo',
                    'price_band': [
                        {
                            'price_band_code': 'PLB',
                            'price_band_desc': 'Plebians',
                            'avail_detail': [{
                                'seatprice': 47.0,
                                'surcharge': 6.5,
                                'full_seatprice': 48.0,
                                'full_surcharge': 7.5,
                            }],
                        },
                    ]
                },
            ],
        }
        details = AvailabilityDetails.from_api_data(data)
        assert len(details) == 1

        assert details[0].seatprice == 47.0
        assert details[0].surcharge == 6.5
        assert details[0].full_seatprice == 48.0
        assert details[0].full_surcharge == 7.5
        assert details[0].combined_price() == 53.5
        assert details[0].combined_full_price() == 55.5

    def test_from_api_data_adds_currency(self):
        data = {
            'ticket_type': [
                {
                    'ticket_type_code': 'FOO',
                    'ticket_type_desc': 'Foo',
                    'price_band': [
                        {
                            'price_band_code': 'PLB',
                            'price_band_desc': 'Plebians',
                            'avail_detail': [{
                                'avail_currency_code': 'gbp',
                            }],
                        },
                    ]
                },
            ],
        }
        details = AvailabilityDetails.from_api_data(data)
        assert len(details) == 1

        assert details[0].currency == 'gbp'

    def test_from_api_data_adds_cached_number_available(self):
        data = {
            'ticket_type': [
                {
                    'ticket_type_code': 'FOO',
                    'ticket_type_desc': 'Foo',
                    'price_band': [
                        {
                            'price_band_code': 'PLB',
                            'price_band_desc': 'Plebians',
                            'avail_detail': [{
                                'cached_number_available': 4,
                            }],
                        },
                    ]
                },
            ],
        }
        details = AvailabilityDetails.from_api_data(data)
        assert len(details) == 1

        assert details[0].cached_number_available == 4

    def test_from_api_data_adds_first_last_dates(self):
        data = {
            'ticket_type': [
                {
                    'ticket_type_code': 'FOO',
                    'ticket_type_desc': 'Foo',
                    'price_band': [
                        {
                            'price_band_code': 'PLB',
                            'price_band_desc': 'Plebians',
                            'avail_detail': [{
                                'available_dates': {
                                    'first_yyyymmdd': '20161129',
                                    'last_yyyymmdd': '20170621',
                                }
                            }],
                        },
                    ]
                },
            ],
        }
        details = AvailabilityDetails.from_api_data(data)
        assert len(details) == 1

        assert details[0].first_date == datetime.date(2016, 11, 29)
        assert details[0].last_date == datetime.date(2017, 6, 21)

    def test_from_api_data_copes_with_zero_dates(self):
        data = {
            'ticket_type': [
                {
                    'ticket_type_code': 'FOO',
                    'ticket_type_desc': 'Foo',
                    'price_band': [
                        {
                            'price_band_code': 'PLB',
                            'price_band_desc': 'Plebians',
                            'avail_detail': [{
                                'available_dates': {
                                    'first_yyyymmdd': '00000000',
                                    'last_yyyymmdd': '00000000',
                                }
                            }],
                        },
                    ]
                },
            ],
        }
        details = AvailabilityDetails.from_api_data(data)
        assert len(details) == 1

        assert details[0].first_date is None
        assert details[0].last_date is None

    def test_from_api_data_adds_calendar_masks(self):
        data = {
            'ticket_type': [
                {
                    'ticket_type_code': 'FOO',
                    'ticket_type_desc': 'Foo',
                    'price_band': [
                        {
                            'price_band_code': 'PLB',
                            'price_band_desc': 'Plebians',
                            'avail_detail': [{
                                'available_dates': {
                                    'year_2016': {
                                        'dec_bitmask': 1065287163,
                                        'nov_bitmask': 536870912
                                    },
                                    'year_2017': {
                                        'feb_bitmask': 251526135,
                                        'jan_bitmask': 2012209087,
                                        'mar_bitmask': 519961591
                                    }
                                }
                            }],
                        },
                    ]
                },
            ],
        }
        details = AvailabilityDetails.from_api_data(data)
        assert len(details) == 1

        assert details[0]._calendar_masks == {
            2016: {
                12: 1065287163,
                11: 536870912
            },
            2017: {
                2: 251526135,
                1: 2012209087,
                3: 519961591
            },
        }

    def test_from_api_data_adds_weekday_masks(self):
        data = {
            'ticket_type': [
                {
                    'ticket_type_code': 'FOO',
                    'ticket_type_desc': 'Foo',
                    'price_band': [
                        {
                            'price_band_code': 'PLB',
                            'price_band_desc': 'Plebians',
                            'avail_detail': [{
                                'available_weekdays_bitmask': 63,
                            }],
                        },
                    ]
                },
            ],
        }
        details = AvailabilityDetails.from_api_data(data)
        assert len(details) == 1

        assert details[0]._weekday_mask == 63

    def test_from_api_data_adds_valid_quantities(self):
        data = {
            'ticket_type': [
                {
                    'ticket_type_code': 'FOO',
                    'ticket_type_desc': 'Foo',
                    'price_band': [
                        {
                            'price_band_code': 'PLB',
                            'price_band_desc': 'Plebians',
                            'avail_detail': [{
                                'valid_quantities': [2, 3, 4],
                            }],
                        },
                    ]
                },
            ],
        }
        details = AvailabilityDetails.from_api_data(data)
        assert len(details) == 1

        assert details[0].valid_quantities == [2, 3, 4]

    def test_is_available_with_valid_year(self, avail_details):
        assert avail_details.is_available(2016) is True

    def test_is_available_with_invalid_year(self, avail_details):
        assert avail_details.is_available(2019) is False

    def test_is_available_fails_with_year_and_day_but_not_month(self, avail_details):
        with pytest.raises(ValueError):
            avail_details.is_available(2016, day=23)

    def test_is_available_with_valid_year_month_combo(self, avail_details):
        assert avail_details.is_available(2016, 12) is True

    def test_is_available_with_invalid_year_month_combo(self, avail_details):
        assert avail_details.is_available(2017, 6) is False

    def test_is_available_with_valid_year_month_day_combo(self, avail_details):
        """
        mask is 1065287163 for dec 2016, this is 0b111111011111101111110111111011
        each bit right to left is a day of the month, i.e we have availbility on
        the 1st and 2nd and 4th, but not on the 3rd, 10th or 17th.
        """
        assert avail_details.is_available(2016, 12, 1) is True
        assert avail_details.is_available(2016, 12, 2) is True
        assert avail_details.is_available(2016, 12, 4) is True

    def test_is_available_with_invalid_year_month_day_combo(self, avail_details):
        """
        mask is 1065287163 for dec 2016, this is 0b111111011111101111110111111011
        each bit right to left is a day of the month, i.e we have availbility on
        the 1st and 2nd and 4th, but not on the 3rd, 10th, 17th.
        """
        assert avail_details.is_available(2016, 12, 3) is False
        assert avail_details.is_available(2016, 12, 10) is False
        assert avail_details.is_available(2016, 12, 17) is False

    def test_on_weekday(self):
        """
        mask is 7 bits representing the 7 days of the week. it's read right to
        left with the right most being sunday and the left most being saturday
        """
        # 23 = 0b10111 (VALID = sun, mon, tues, thurs; INVALID = wed, fri, sat)
        details = AvailabilityDetails(weekday_mask=23)

        # 0 is monday
        assert details.on_weekday(0) is True
        # 1 is tuesday
        assert details.on_weekday(1) is True
        # 2 is wednesday
        assert details.on_weekday(2) is False
        # 3 is thursday
        assert details.on_weekday(3) is True
        # 4 is friday
        assert details.on_weekday(4) is False
        # 5 is saturday
        assert details.on_weekday(5) is False
        # 6 is sunday
        assert details.on_weekday(6) is True



from pyticketswitch.country import Country


class TestCountry:

    def test_from_api_data(self):
        data = {
            'country_code': 'uk',
            'country_desc': 'United Kingdom'
        }

        country = Country.from_api_data(data)

        assert country.code == 'uk'
        assert country.description == 'United Kingdom'



from pyticketswitch.debitor import Debitor


class TestDebitor:

    def test_from_api_data(self):

        data = {
            "debitor_desc": "Stripe",
            "debitor_integration_data": {
                'publishable_key': 'abc123',
            },
            "debitor_name": "stripe_gbp",
            "debitor_type": "stripe",
            "debitor_aggregation_key": "abcd1234"
        }

        debitor = Debitor.from_api_data(data)

        assert debitor.type == 'stripe'
        assert debitor.name == 'stripe_gbp'
        assert debitor.description == 'Stripe'
        assert debitor.integration_data == {
            'publishable_key': 'abc123',
        }
        assert debitor.aggregation_key == 'abcd1234'



import datetime
from dateutil.tz import tzoffset
from pyticketswitch.status import Status
from pyticketswitch.address import Address
from pyticketswitch.trolley import Trolley


class TestStatus:

    ZULU = tzoffset('ZULU', 0)

    def test_from_api_data(self):

        data = {
            "allowed_countries": {
                "ad": "Andorra",
                "ae": "United Arab Emirates",
            },
            "accepted_payment_cards": {
                "amex": "American Express",
                "visa": "Visa"
            },
            "can_edit_address": True,
            "language_list": ["en-gb", "en", "en-us", "nl"],
            "minutes_left_on_reserve": 57.5,
            "needs_agent_reference": True,
            "needs_email_address": True,
            "needs_payment_card": True,
            "supports_billing_address": True,
            "remote_site": 'foobar.example.com',
            "prefilled_address": {
                'addr_line_one': '123 fakestreet',
            },
            "reserve_iso8601_date_and_time": "2016-11-9T18:19:27Z",
            "purchase_iso8601_date_and_time": "2016-11-11T12:34:49Z",
            "transaction_status": "reserved",
            "trolley_contents": {
                "transaction_uuid": "6a2f2e03-08e8-11e7-a86b-d0bf9c45f5c0",
            },
            "reserve_user": {"user_id": "foobar"},
            "pending_callout": {"bundle_source_code": "ext_test0"},
            'customer': {
                "addr_line_one": "Metro Building",
                "addr_line_one_latin": "METRO BUILDING",
                "addr_line_two": "1 Butterwick",
                "addr_line_two_latin": "1 BUTTERWICK",
                "agent_ref": "foobar123",
                "country": "United Kingdom",
                "country_code": "uk",
                "country_latin": "UNITED KINGDOM",
                "county": "London",
                "county_latin": "LONDON",
                "dp_supplier": True,
                "dp_user": True,
                "dp_world": True,
                "email_addr": "tester@gmail.com",
                "first_name": "Test",
                "first_name_latin": "TEST",
                "home_phone": "0203 137 7420",
                "work_phone": "079888888888",
                "phone": "079000000000",
                "initials": "l.o.l",
                "initials_latin": "L.O.L",
                "last_name": "Tester",
                "last_name_latin": "TESTER",
                "postcode": "w6 8dl",
                "postcode_latin": "W6 8DL",
                "suffix": "esq",
                "suffix_latin": "ESQ",
                "title": "rt.hon",
                "title_latin": "RT.HON",
                "town": "Hammersmith",
                "town_latin": "HAMMERSMITH",
            }
        }

        status = Status.from_api_data(data)

        assert status.status == 'reserved'
        assert status.reserved_at == datetime.datetime(2016, 11, 9, 18, 19, 27, tzinfo=self.ZULU)
        assert status.purchased_at == datetime.datetime(2016, 11, 11, 12, 34, 49, tzinfo=self.ZULU)
        assert status.external_sale_page is None
        assert status.languages == ['en-gb', 'en', 'en-us', 'nl']
        assert status.remote_site == 'foobar.example.com'
        assert status.trolley.transaction_uuid == '6a2f2e03-08e8-11e7-a86b-d0bf9c45f5c0'
        # assert status.reserve_user.id == 'foobar'
        assert status.needs_payment_card is True
        assert status.needs_email_address is True
        assert status.needs_agent_reference is True
        assert status.can_edit_address is True
        assert status.minutes_left == 57.5
        assert status.supports_billing_address is True

        assert isinstance(status.prefilled_address, Address)
        assert isinstance(status.trolley, Trolley)

        assert len(status.allowed_countries) == 2
        assert status.allowed_countries[0].code == 'ad'
        assert status.allowed_countries[1].code == 'ae'

        assert len(status.accepted_cards) == 2
        assert any(x.code == 'amex' for x in status.accepted_cards)
        assert any(x.code == 'visa' for x in status.accepted_cards)
        assert any(x.description == 'Visa' for x in status.accepted_cards)
        assert any(x.description == 'American Express' for x in status.accepted_cards)

        assert status.pending_callout.code == 'ext_test0'
